<?php

/*
 * Copyright (C) 2017 Manuela v.d.Decken <manuela@isteam.de>
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT OR THIS HEADER
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License 2 for more details.
 *
 * You should have received a copy of the GNU General Public License 2
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Description of dlc.php
 *
 * @package      Addon_download_gallery
 * @copyright    Manuela v.d.Decken <manuela@isteam.de>
 * @author       Manuela v.d.Decken <manuela@isteam.de>
 * @license      GNU General Public License 2.0
 * @version      2.62
 * @revision     $Id: $
 * @since        File available since 21.09.2017
 * @deprecated   no / since 0000/00/00
 * @description  dlc.php delivers the selected file, if the user has permissions,
 *               and increments the download counter. For more about, see info.php
 *               Works with WB from 2.11.x and up
 *
 *               How to use:   dlc.php?id=E09DE1
 *               Where 'file' is the ID-Key representing the record-id of the requested file
 *               and 'id' is the timestamp of last modification of this record.
 *               An record is valid only, if record-id AND modified_when matches the request
 */


// include needed script files
if (!\defined('SYSTEM_RUN')) {require(((\dirname((__DIR__),2))).'/config.php');}

    $sAddonName = basename(__DIR__);
    $sAddonObject = 'dlg';
    $sAddonRel  = '/modules/'.$sAddonName;
    $sAddonUrl  = WB_URL.$sAddonRel;
    $sAddonPath = str_replace(['\\'],['/'],WB_PATH.'/'.$sAddonRel);
    $MediaRel   = MEDIA_DIRECTORY;

    $sAddonTemplateRel = '/'.$sAddonRel.'/templates';
    if (!\function_exists('init_fields')) {require ($sAddonPath.'/functions.php');}

//    require (WB_PATH.'/framework/functions.php');
    $aDlgFile = [];
// validate arguments
    $iProve  = (isset($_GET['id']) ? $_GET['id'] : '');
    if (isset($_GET['id']) && preg_match('/[A-Z0-9]+/',$_GET['id'])) {
        $sFileId = $_GET['id'];
    } else {
        $sErrMsg = 'Illegal character in file ID';
        goto REQUEST_ERR;
    }

// search for and test if file is available and readable
    $sql = 'SELECT * FROM `'.TABLE_PREFIX.'mod_download_gallery_files` '
         . 'WHERE `file_id`='.decodeVar($sFileId).'';
    $oFile = $database->query($sql);
    $sDownloadFile = 'unkown.none';
    $aPathParts = pathinfo($sDownloadFile);
    $iDlFileSize = 0;

    if (($aDlgFile = $oFile->fetchAssoc())){
        if (strpos($aDlgFile['filename'],':/')){
        $iDlFileSize = $aDlgFile['size'];
            $sDownloadFile = $aDlgFile['filename'];
            $sParseFile = parse_url($aDlgFile['link']);
            $sParseFile['path'] = str_replace(MEDIA_DIRECTORY, '', $sParseFile['path']);
        }
        else
        {
            $sDownloadFile = WB_PATH.MEDIA_DIRECTORY.$aDlgFile['link'];
            $orgfile =  $sDownloadFile;
            $iDlFileSize = filesize($sDownloadFile);
        }

    } else {
        $sErrMsg = $database->get_error();
        goto REQUEST_ERR;
	}	
    //

    if (strpos($aDlgFile['filename'],':/')){
        /* nothing to do only prevent normal files*/
    }elseif (!is_readable( $sDownloadFile )) {
        $sErrMsg = 'requested file not available';
        goto REQUEST_ERR;
    } else {
        $iDlFileSize = filesize($sDownloadFile);
    }

// search for the corresponding page record
    $sql = 'SELECT * FROM `'.TABLE_PREFIX.'pages` '
         . 'WHERE `page_id`='.$aDlgFile['page_id'];
    if (!(($oPage = $database->query($sql)) && ($aPage = $oPage->fetchAssoc()))) {
        $sErrMsg = 'corresponding page not available';
        goto REQUEST_ERR;
    }
// check if current user has rights to download this file
    if (!in_array($aPage['visibility'], ['public','hidden'])) {
        $iUserId = intval(isset($_SESSION['USER_ID']) ? $_SESSION['USER_ID'] : 0);
        if ($iUserId && in_array($aPage['visibility'], ['registered','private'])) {
//--- Callback returns true if intersection of both lists is greater then 0 ------------//
            $cbGroupMatch = function($sGroup1, $sGroup2) {
                $aResult = array_intersect(
                    preg_split('/[,; |]/', $sGroup1, -1, PREG_SPLIT_NO_EMPTY),
                    preg_split('/[,; |]/', $sGroup2, -1, PREG_SPLIT_NO_EMPTY)
                );
                return (bool) sizeof($aResult);
            };
//--------------------------------------------------------------------------------------//
// check if current user has viewing rights to this page
            if (
                !$cbGroupMatch($_SESSION['GROUPS_ID'], $aPage['viewing_groups']) &&
                !$cbGroupMatch($iUserId, $aPage['viewing_users']) &&
                $iUserId != 1){
                unset($cbGroupMatch);
                $sErrMsg = 'access denied';
                goto REQUEST_ERR;
            }
            unset($cbGroupMatch);
        } else {
            $sErrMsg = 'access denied';
            goto REQUEST_ERR;
        }
    }
// increment download counter:
    $sql = 'UPDATE `'.TABLE_PREFIX.'mod_download_gallery_files` '
         . 'SET `dlcount`=`dlcount`+1 '
         . 'WHERE `file_id`='.decodeVar($sFileId);
    if ($database->query($sql))
    {
        if (!class_exists('httpdownload')){require($sAddonPath.'/lib/class.httpdownload.php');}
        $object = new httpdownload;

        if (in_array($aDlgFile['extension'],['pdf'])){
            $object->set_byfile($sDownloadFile);
        } elseif (isset($sParseFile['scheme'])){
            $object->set_byurl($aDlgFile['filename']);
        } else {
            $object->set_byfile($sDownloadFile); //Download from a file
        }
         $object->use_resume = true; //Enable Resume Mode
         $object->download(); //Download File
        exit;
    } else {
        $sErrMsg = 'unable to modify download counter';
    }
REQUEST_ERR:
// send error messages
    echo $sErrMsg;
    exit;